@@ -4,8 +4,7 @@
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
-#ifndef LINGLONG_BOX_SRC_CONTAINER_MOUNT_HOST_MOUNT_H_
-#define LINGLONG_BOX_SRC_CONTAINER_MOUNT_HOST_MOUNT_H_
+#pragma once
 
 #include "util/oci_runtime.h"
 
@@ -31,5 +30,3 @@ private:
 };
 
 } // namespace linglong
-
-#endif /* LINGLONG_BOX_SRC_CONTAINER_MOUNT_HOST_MOUNT_H_ */
