@@ -4,12 +4,9 @@
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
-#ifndef LINGLONG_BOX_SRC_UTIL_MACRO_H_
-#define LINGLONG_BOX_SRC_UTIL_MACRO_H_
+#pragma once
 
 #define LINGLONG 118
 
 #define LL_VAL(str) #str
 #define LL_TOSTRING(str) LL_VAL(str)
-
-#endif /* LINGLONG_BOX_SRC_UTIL_MACRO_H_ */
