@@ -5,10 +5,10 @@
  */
 
 #pragma once
-
 #include "common.h"
 
 #include <optional>
+#include <string_view>
 
 namespace linglong {
 
@@ -22,6 +22,8 @@ int Wait(const int pid);
 int WaitAll();
 int WaitAllUntil(const int pid);
 
+int strToSig(std::string_view str) noexcept;
+
 } // namespace util
 
 } // namespace linglong
