@@ -134,8 +134,8 @@ parseProjectConfig(const QString &filename)
     if (!project) {
         return project;
     }
-    auto version = linglong::package::Version(QString::fromStdString(project->package.version));
-    if (!version.tweak) {
+    auto version = linglong::package::Version::parse(QString::fromStdString(project->package.version));
+    if (!version || !version->tweak) {
         return LINGLONG_ERR("Please ensure the package.version number has three parts formatted as "
                             "'MAJOR.MINOR.PATCH.TWEAK'");
     }
@@ -286,7 +286,7 @@ You can report bugs to the linyaps team under this project: https://github.com/O
 
     // build export
     bool layerMode = false;
-    linglong::builder::ExportOption ExportOption{ .exportDevelop = false, .exportI18n = true };
+    linglong::builder::ExportOption ExportOption{ .exportI18n = true };
     auto *buildExport = commandParser.add_subcommand("export", _("Export to linyaps layer or uab"));
     buildExport->usage(_("Usage: ll-builder export [OPTIONS]"));
 
@@ -311,6 +311,8 @@ You can report bugs to the linyaps team under this project: https://github.com/O
       ->type_name("FILE")
       ->check(CLI::ExistingFile)
       ->excludes(layerFlag, fullOpt);
+    buildExport->add_flag("--no-develop", ExportOption.noExportDevelop, _("Don't export the develop module"))
+        ->needs(layerFlag);
 
     // build push
     std::string pushModule;
@@ -490,6 +492,17 @@ You can report bugs to the linyaps team under this project: https://github.com/O
         return -1;
     }
 
+    // set GIO_USE_VFS to local, avoid glib start thread
+    char *oldEnvGioUseVFS = getenv("GIO_USE_VFS");
+    if (-1 == setenv("GIO_USE_VFS", "local", 1)) {
+        qWarning() << "failed to GIO_USE_VFS to local" << errno;
+    }
+
+    char *oldEnvRemoteIgnore = getenv("GVFS_REMOTE_VOLUME_MONITOR_IGNORE");
+    if (-1 == setenv("GVFS_REMOTE_VOLUME_MONITOR_IGNORE", "1", 1)) {
+        qWarning() << "failed to GVFS_REMOTE_VOLUME_MONITOR_IGNORE to local" << errno;
+    }
+
     auto result = linglong::repo::tryMigrate(builderCfg->repo, *repoCfg);
     if (result == linglong::repo::MigrateResult::Failed) {
         auto pathTemp = (std::filesystem::path{ builderCfg->repo }.parent_path()
@@ -527,16 +540,10 @@ You can report bugs to the linyaps team under this project: https://github.com/O
         return -1;
     }
 
-    // set GIO_USE_VFS to local, avoid glib start thread
-    char *oldEnv = getenv("GIO_USE_VFS");
-    if (-1 == setenv("GIO_USE_VFS", "local", 1)) {
-        qWarning() << "failed to GIO_USE_VFS to local" << errno;
-    }
-
     linglong::repo::OSTreeRepo repo(repoRoot, *repoCfg, clientFactory);
 
-    if (oldEnv) {
-        if (-1 == setenv("GIO_USE_VFS", oldEnv, 1)) {
+    if (oldEnvGioUseVFS) {
+        if (-1 == setenv("GIO_USE_VFS", oldEnvGioUseVFS, 1)) {
             qWarning() << "failed to restore GIO_USE_VFS" << errno;
         }
     } else {
@@ -545,6 +552,16 @@ You can report bugs to the linyaps team under this project: https://github.com/O
         }
     }
 
+    if (oldEnvRemoteIgnore) {
+        if (-1 == setenv("GVFS_REMOTE_VOLUME_MONITOR_IGNORE", oldEnvRemoteIgnore, 1)) {
+            qWarning() << "failed to restore GVFS_REMOTE_VOLUME_MONITOR_IGNORE" << errno;
+        }
+    } else {
+        if (-1 == unsetenv("GVFS_REMOTE_VOLUME_MONITOR_IGNORE")) {
+            qWarning() << "failed to restore GVFS_REMOTE_VOLUME_MONITOR_IGNORE" << errno;
+        }
+    }
+
     // if user use old opt(--exec) passing parameters, pass it to the new commands;
     if (newCommands.empty() && !oldCommands.empty()) {
         newCommands = oldCommands;
@@ -814,7 +831,7 @@ You can report bugs to the linyaps team under this project: https://github.com/O
             if (!ExportOption.compressor.empty()) {
                 compressor = ExportOption.compressor.c_str();
             }
-            auto result = builder.exportLayer(QDir::currentPath(), compressor);
+            auto result = builder.exportLayer(QDir::currentPath(), compressor, ExportOption.noExportDevelop);
             if (!result) {
                 qCritical() << result.error();
                 return -1;
