@@ -30,17 +30,6 @@ LayerPackager::LayerPackager(const QDir &workDir)
 
 LayerPackager::~LayerPackager()
 {
-    for (const auto &info : this->workDir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot)) {
-        if (!info.isDir()) {
-            continue;
-        }
-
-        auto ret = utils::command::Exec("umount", { info.absoluteFilePath() });
-        if (!ret) {
-            qCritical() << ret.error();
-        }
-    }
-
     if (this->workDir.removeRecursively()) {
         return;
     }
