@@ -2,8 +2,7 @@
 //
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
-#ifndef UAB_FILE_H
-#define UAB_FILE_H
+#pragma once
 
 #include "linglong/api/types/v1/UabMetaInfo.hpp"
 #include "linglong/utils/error/error.h"
@@ -12,9 +11,10 @@
 #include <libelf.h>
 
 #include <QDir>
-#include <QScopedPointer>
 #include <QString>
 
+#include <filesystem>
+
 namespace linglong::package {
 
 class UABFile : public QFile
@@ -26,7 +26,10 @@ public:
     ~UABFile() override;
 
     utils::error::Result<bool> verify() noexcept;
-    utils::error::Result<QDir> mountUab() noexcept;
+    utils::error::Result<std::filesystem::path> mountUab() noexcept;
+
+    // this method will extract sign data to a temporary directory, caller should remove it
+    utils::error::Result<std::filesystem::path> extractSignData() noexcept;
     [[nodiscard]] utils::error::Result<std::reference_wrapper<const api::types::v1::UabMetaInfo>>
     getMetaInfo() noexcept;
 
@@ -37,9 +40,7 @@ private:
 
     Elf *e{ nullptr };
     std::unique_ptr<api::types::v1::UabMetaInfo> metaInfo{ nullptr };
-    QString mountPoint;
+    std::filesystem::path mountPoint;
 };
 
 } // namespace linglong::package
-
-#endif
