@@ -69,14 +69,19 @@ public:
 
     utils::error::Result<void> setIcon(const QFileInfo &icon) noexcept;
     utils::error::Result<void> appendLayer(const LayerDir &layer) noexcept;
-    utils::error::Result<void> pack(const QString &uabFilename) noexcept;
+    utils::error::Result<void> pack(const QString &uabFilename, bool onlyApp) noexcept;
     utils::error::Result<void> exclude(const std::vector<std::string> &files) noexcept;
     utils::error::Result<void> include(const std::vector<std::string> &files) noexcept;
+    utils::error::Result<void> loadBlackList() noexcept;
+    utils::error::Result<void> loadNeededFiles() noexcept;
+    utils::error::Result<void> setLoader(const QString &loader) noexcept;
+    utils::error::Result<void> setCompressor(const QString &compressor) noexcept;
 
 private:
     [[nodiscard]] utils::error::Result<void> packIcon() noexcept;
-    [[nodiscard]] utils::error::Result<void> packBundle() noexcept;
-    [[nodiscard]] utils::error::Result<void> prepareBundle(const QDir &bundleDir) noexcept;
+    [[nodiscard]] utils::error::Result<void> packBundle(bool onlyApp) noexcept;
+    [[nodiscard]] utils::error::Result<void> prepareBundle(const QDir &bundleDir,
+                                                           bool onlyApp) noexcept;
     [[nodiscard]] utils::error::Result<void> packMetaInfo() noexcept;
     [[nodiscard]] utils::error::Result<std::pair<bool, std::unordered_set<std::string>>>
     filteringFiles(const LayerDir &layer) const noexcept;
@@ -85,8 +90,13 @@ private:
     QList<LayerDir> layers;
     std::unordered_set<std::string> excludeFiles;
     std::unordered_set<std::string> includeFiles;
+    std::unordered_set<std::string> neededFiles;
+    std::unordered_set<std::string> blackList;
     std::optional<QFileInfo> icon{ std::nullopt };
     api::types::v1::UabMetaInfo meta;
     QDir buildDir;
+    std::filesystem::path workDir;
+    QString loader;
+    QString compressor = "lz4";
 };
 } // namespace linglong::package
