@@ -4,8 +4,7 @@
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
-#ifndef LINGLONG_PACKAGE_VERSION_H_
-#define LINGLONG_PACKAGE_VERSION_H_
+#pragma once
 
 #include "linglong/utils/error/error.h"
 
@@ -37,5 +36,3 @@ public:
     QString toString() const noexcept;
 };
 } // namespace linglong::package
-
-#endif
