@@ -124,10 +124,11 @@ PackageManager::PackageManager(linglong::repo::OSTreeRepo &repo,
 
     auto *timer = new QTimer(this);
     timer->setInterval(deferredTimeOut);
-    timer->callOnTimeout([this, timer] {
+    connect(timer, &QTimer::timeout, [this, timer] {
         this->deferredUninstall();
         timer->start();
     });
+
     timer->start();
 }
 
@@ -646,7 +647,7 @@ QVariantMap PackageManager::installFromLayer(const QDBusUnixFileDescriptor &fd,
 
           auto unmountLayer = utils::finally::finally([mountPoint = layerDir->absolutePath()] {
               if (QFileInfo::exists(mountPoint)) {
-                  auto ret = utils::command::Exec("umount", { mountPoint });
+                  auto ret = utils::command::Exec("fusermount", { "-z", "-u", mountPoint });
                   if (!ret) {
                       qCritical() << "failed to umount " << mountPoint
                                   << ", please umount it manually";
@@ -2368,6 +2369,25 @@ auto PackageManager::GenerateCache(const QString &reference) noexcept -> QVarian
     auto jobID = QUuid::createUuid().toString();
     m_generator_queue.runTask([this, jobID, ref]() {
         qInfo() << "Generate cache for:" << ref.toString();
+
+        const auto &appLayerItem = this->repo.getLayerItem(ref);
+        if (!appLayerItem) {
+            qWarning() << "failed to get app layer item" << appLayerItem.error();
+            return;
+        }
+        auto appCache = std::filesystem::path(LINGLONG_ROOT) / "cache" / appLayerItem->commit;
+
+        std::error_code ec;
+        if (std::filesystem::exists(appCache, ec)) {
+            qInfo() << "The cache has been generated.";
+            return;
+        }
+
+        if (ec) {
+            qCritical() << "failed to get app cache" << ec.message().c_str() << ec.value();
+            return;
+        }
+
         auto ret = this->generateCache(ref);
         if (!ret) {
             qCritical() << "failed to generate cache for:" << ref.toString();
