@@ -4,7 +4,7 @@
 
 #pragma once
 
-#include "linglong/api/types/v1/RepoConfig.hpp"
+#include "linglong/api/types/v1/RepoConfigV2.hpp"
 
 #include <filesystem>
 
@@ -12,5 +12,5 @@ namespace linglong::repo {
 enum class MigrateResult { Success, Failed, NoChange };
 
 MigrateResult tryMigrate(const std::filesystem::path &root,
-                         const linglong::api::types::v1::RepoConfig &cfg) noexcept;
+                         const linglong::api::types::v1::RepoConfigV2 &cfg) noexcept;
 } // namespace linglong::repo
