@@ -4,8 +4,7 @@
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
-#ifndef LINGLONG_UTILS_COMMAND_ENV_H_
-#define LINGLONG_UTILS_COMMAND_ENV_H_
+#pragma once
 
 #include "linglong/utils/error/error.h"
 
@@ -18,5 +17,3 @@ QStringList getUserEnv(const QStringList &filters);
 extern const QStringList envList;
 
 } // namespace linglong::utils::command
-
-#endif
