@@ -4,8 +4,7 @@
  * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
-#ifndef LINGLONG_UTILS_STD_HELPER_QDEBUG_HELPER_H_
-#define LINGLONG_UTILS_STD_HELPER_QDEBUG_HELPER_H_
+#pragma once
 
 #include <QDebug>
 
@@ -14,5 +13,3 @@ inline auto operator<<(QDebug &out, const std::string &str) -> QDebug &
     out << QString::fromStdString(str);
     return out;
 }
-
-#endif
